/*    icmp_type_lookup.c
 *    Copyright 2002-2004 Kirby Kuehl (vacuum@users.sourceforge.net)
 *    This file is part of pixilate
 *    http://winfingerprint.sourceforge.net/pixilate.php
 *
 *    pixilate is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    pixilate is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *    You should have received a copy of the GNU General Public License
 *    along with pixilate; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pixilate.h"
/* icmp_type_lookup() attempts to match a character string to a
 * set of literals. If no match is found, convert the string to an
 * int with atoi(). icmp_type_lookup() returns the icmp type as an int.
 */
int cisco_icmp_type_lookup(char *literal, struct acl *pacl)
{
   /* Sanity check */
   if(pacl->protocol != IPPROTO_ICMP)
   {
      fprintf(stdout,"NON-ICMP packet with icmp_type\n");
      return(-1);
   }
   
   if(strncmp(literal,"echo-reply",10) == 0)
      pacl->icmp_type = 0;
   else if(strncmp(literal,"unreachable",11) == 0)
      pacl->icmp_type = 3;
   else if(strncmp(literal,"source-quench",13) == 0)
      pacl->icmp_type = 4;
   else if(strncmp(literal,"redirect",8) == 0)
      pacl->icmp_type = 5;
   else if(strncmp(literal,"alternate-address",17) == 0)
      pacl->icmp_type = 6;
   else if(strncmp(literal,"echo",4) == 0)
      pacl->icmp_type = 8;
   else if(strncmp(literal,"router-advertisement",20) == 0)
      pacl->icmp_type = 9;
   else if(strncmp(literal,"router-solicitation",19) == 0)
      pacl->icmp_type = 10;
   else if(strncmp(literal,"time-exceeded",13) == 0)
      pacl->icmp_type = 11;
   else if(strncmp(literal,"parameter-problem",17) == 0)
      pacl->icmp_type = 12;
   else if(strncmp(literal,"timestamp-reply",15) == 0)
      pacl->icmp_type = 13;
   else if(strncmp(literal,"timestamp-request",17) == 0)
      pacl->icmp_type = 14;
   else if(strncmp(literal,"information-request",19) == 0)
      pacl->icmp_type = 15;
   else if(strncmp(literal,"information-reply",17) == 0)
      pacl->icmp_type = 16;
   else if(strncmp(literal,"mask-request",12) == 0)
      pacl->icmp_type = 17;
   else if(strncmp(literal,"mask-reply",10) == 0)
      pacl->icmp_type = 18;
   else if(strncmp(literal,"conversion-error",16) == 0)
      pacl->icmp_type = 31;
   else if(strncmp(literal,"mobile-redirect",15) == 0)
      pacl->icmp_type = 32;
   else 
      pacl->icmp_type = atoi(literal);
   
  return(pacl->icmp_type);
}

