/*    packetize.c
 *    Copyright 2002-2004 Kirby Kuehl (vacuum@users.sourceforge.net)
 *    This file is part of pixilate
 *    http://winfingerprint.sourceforge.net/pixilate.php
 *
 *    pixilate is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    pixilate is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *    You should have received a copy of the GNU General Public License
 *    along with pixilate; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "pixilate.h"

int packetize(struct acl *pacl, libnet_t *l)
{
    libnet_ptag_t ip, icmp, tcp, tcp_ip, udp, udp_ip, igmp, igmp_ip, other;
    int icmp_header_len = 0,
        c               = 0,
        unsupported     = 0;
    tcp    = LIBNET_PTAG_INITIALIZER;
    tcp_ip = LIBNET_PTAG_INITIALIZER;
    udp    = LIBNET_PTAG_INITIALIZER;
    udp_ip = LIBNET_PTAG_INITIALIZER;
    icmp   = LIBNET_PTAG_INITIALIZER;
    ip     = LIBNET_PTAG_INITIALIZER;
    igmp   = LIBNET_PTAG_INITIALIZER;
    igmp_ip= LIBNET_PTAG_INITIALIZER;
    other  = LIBNET_PTAG_INITIALIZER;


    if(pacl->rule & IPSEC_RULE)
    {
        fprintf(stdout,
            "Although the ipsec packet building functions exist within "
            "libnet, the necessary cryptographic functions do not. "
            "For *now*, this packet will not be built.\n");
        return(-1);
    }
    else if(pacl->rule & ICMP_RULE)
    {
        switch(pacl->icmp_type)
        {
            case 0: // icmp type 0 literal echo-reply
                icmp = libnet_build_icmpv4_echo(
                    ICMP_ECHOREPLY, //type
                    0,              //code
                    0,              // 0 lets libnet calculate checksum
                    pacl->id,       //id
                    pacl->seq,      //seq
                    pacl->payload,  // payload
                    pacl->payload_s,// sizeof payload
                    l,              // libnet handle
                    0);             // create new  ptag
                if(icmp == -1)
                    fprintf(stderr,
                        "Unable to build icmp echo reply packet :%s\n",
                    libnet_geterror(l));
                else
                    fprintf(stdout,"Created ICMP echo reply packet.\n");

                icmp_header_len = LIBNET_ICMPV4_ECHO_H;
                break;
            case 3: /* icmp type 3 literal unreachable */
                icmp = libnet_build_icmpv4_unreach(
                    ICMP_UNREACH,             /* type */ 
                    ICMP_UNREACH_PORT,        /* code */
                    0,                        /* 0 libnet calculate checksum */
                    pacl->payload,            /* payload */ 
                    pacl->payload_s,          /* payload size */
                    l,                        /* libnet handle */
                    0);                       /* 0 create new ptag */
                if(icmp == -1)
                    fprintf(stderr,"Unable to build icmp unreach packet :%s\n",
                        libnet_geterror(l));
                else
                   fprintf(stdout,"Created ICMP unreach packet.\n");

                icmp_header_len = LIBNET_ICMPV4_UNREACH_H;
                break;
            case 5: /* icmp type 5 literal redirect */
                icmp =  libnet_build_icmpv4_redirect(
                    ICMP_REDIRECT,          /* type */  
                    ICMP_UNREACH_HOST,      /* code */ 
                    0,              /*0 lets libnet calculate checksum */
                    pacl->gateway_addr,     /* gateway */
                    pacl->payload,          /* payload */
                    pacl->payload_s,        /* payload size */
                    l,                      /* libnet handle */
                    0);                     /* 0 create new ptag */
                icmp_header_len = LIBNET_ICMPV4_REDIRECT_H;
                break;
            case 8: // icmp type 8 literal echo
                icmp = libnet_build_icmpv4_echo(
                    ICMP_ECHO,       //type
                    0,               //code
                    0,               // checksum 
                    pacl->id,        //  id
                    pacl->seq,       // sequence
                    pacl->payload,   // payload
                    pacl->payload_s, // payload size
                    l,               // libnet handle
                    0);              // create new ptag
                if(icmp == -1)
                    fprintf(stderr,"Unable to build icmp echo packet :%s\n",
                        libnet_geterror(l));
                else
                    fprintf(stdout,"Created icmp echo packet.\n");

                icmp_header_len = LIBNET_ICMPV4_ECHO_H;
                break;
            case 11: /* icmp type 11 literal time-exceeded */
                icmp = libnet_build_icmpv4_timeexceed(
                    ICMP_TIMXCEED,          /* type */ 
                    ICMP_TIMXCEED_INTRANS,  /* code */
                    0,                  /* 0 lets libnet calculate checksum */
                    pacl->payload,          /* payload */
                    pacl->payload_s,        /* payload size */
                    l,                      /* libnet handle */
                    0);                     /* create new ptag */
                icmp_header_len = LIBNET_ICMPV4_TIMXCEED_H;
                break;
            case 13: // icmp type 13 literal timestamp-reply
                icmp =  libnet_build_icmpv4_timestamp(
                    ICMP_TSTAMPREPLY, //type
                    0, //code
                    0,  // 0 lets libnet calculate checksum
                    pacl->id,                   // id 
                    pacl->seq,                  // sequence number 
                    1000,                       // otime 
                    2000,                       // rtime 
                    3000,                       // ttime 
                    pacl->payload,              // payload 
                    pacl->payload_s,            // payload size 
                    l,                          // libnet handle 
                    0);                         // create new ptag
                if(icmp == -1)
                    fprintf(stderr,"Unable to build icmp timestamp reply packet :%s\n",
                        libnet_geterror(l));
                else
                    fprintf(stdout,"Created icmp timestamp reply packet.\n");

                icmp_header_len = LIBNET_ICMPV4_TS_H;
                break;
            case 14: // icmp type 14 literal timestamp-request
                icmp = libnet_build_icmpv4_timestamp(
                    ICMP_TSTAMP,           //type
                    0,                     //code
                    0,                     // 0 lets libnet calculate checksum 
                    pacl->id,              // id 
                    pacl->seq,             // sequence number 
                    1000,                  // otime 
                    2000,                  // rtime 
                    3000,                  // ttime 
                    pacl->payload,         // payload 
                    pacl->payload_s,       // payload size 
                    l,                     // libnet handle
                    0);                    // create new ptag
                if(icmp == -1)
                    fprintf(stderr,
                        "Unable to build icmp timestamp request packet :%s\n",
                        libnet_geterror(l));
                else
                    fprintf(stdout,"Created icmp timestamp request packet.\n");
          
                icmp_header_len = LIBNET_ICMPV4_TS_H;
                break;
            case 17: // icmp type 17 literal mask-request
                icmp = libnet_build_icmpv4_mask(
                    ICMP_MASKREQ,    // type 
                    0,               // code 
                    0,               // 0 lets libnet calculate checksum
                    pacl->id,        // identification number
                    pacl->seq,       // sequence number
                    0xffffffff,      // subnet mask
                    pacl->payload,   // payload 
                    pacl-> payload_s,// payload size 
                    l,               // libnet handle
                    0);              // create new ptag
                if(icmp == -1)
                    fprintf(stderr,
                        "Unable to build icmp mask request packet :%s\n",
                        libnet_geterror(l));
                else
                    fprintf(stdout,"Created icmp mask request packet.\n");

                icmp_header_len = LIBNET_ICMPV4_MASK_H;
                break;
            case 18: // icmp type 18 literal mask-reply
                icmp =  libnet_build_icmpv4_mask(
                    ICMP_MASKREPLY,  // type 
                    0,               // code 
                    0,               // 0 lets libnet calculate checksum
                    pacl->id,        // identification number
                    pacl->seq,       // sequence number
                    0xffffffff,      // subnet mask
                    pacl->payload,   // payload 
                    pacl-> payload_s,// payload size 
                    l,               // libnet handle
                    0);              // create new ptag
                if(icmp == -1)
                    fprintf(stderr,
                        "Unable to build icmp mask reply packet :%s\n",
                        libnet_geterror(l));
                else
                    fprintf(stdout,"Created icmp mask reply packet.\n");
                
                icmp_header_len = LIBNET_ICMPV4_MASK_H;
                 break;
       
             default:
                fprintf(stdout,
                    "Libnet is unable to build icmp type %d packets.\n",
                    pacl->icmp_type);
                return(-1); 
                break;
        }

        ip = libnet_build_ipv4(
            (LIBNET_IPV4_H + icmp_header_len + pacl->payload_s), // length
            IPTOS_LOWDELAY | IPTOS_THROUGHPUT,   // Type of Service
            pacl->id,               // IP ID
            0,                      // IP fragment bits and offset
            64,                     // TTL
            IPPROTO_ICMP,           // protocol
            0,                      // checksum
            pacl->source_addr,      // source ip 
            pacl->destination_addr, // destination ip
            NULL,                   // payload
            0,                      // payload size
            l,                      // libnet handle
            0);                     // create new ptag
 
        if(ip == -1)
        {
            fprintf(stderr,"Can't build IP header: %s\n",libnet_geterror(l));
            return(-1);
        }
        c = libnet_write(l);
        if (c == -1)
        {
            fprintf(stderr, "Write error: %s\n", libnet_geterror(l));
            return(-1);
        }
  
        if(pacl->options & VERBOSE) 
            stats(c,pacl,l);
        return(0);
    }
    else
    {
        unsupported = 0;
        switch(pacl->protocol)
        {
            case IPPROTO_IP:  // 0 since protocol was 'ip' which in pix acl
                              // terms means any internet protocol
                              // (TCP/UDP/ICMP/etc) lets send a
                              // tcp packet by simply falling through the case
                              // statement
            case IPPROTO_TCP: // 6
               tcp = libnet_build_tcp(
                  pacl->source_port,       // source port
                  pacl->destination_port,  // destination port
                  pacl->seq,               // sequence number 
                  0x02020202,              // acknowledgement number
                  TH_SYN,                  // control bits 
                  32767,                   // Advertised window size
                  0,                       // checksum
                  0,                       // urgent pointer
                  pacl->proto_h_len,       // length ofprotocol header
                  pacl->payload,           // data
                  pacl->payload_s,         // payload length
                  l,
                  0);
               if(tcp == -1)
                  fprintf(stderr,"Unable to build TCP header.\n");
               tcp_ip = libnet_build_ipv4(
                  (LIBNET_IPV4_H + pacl->protocol + pacl->payload_s), // length
                  IPTOS_LOWDELAY | IPTOS_THROUGHPUT,   // Type of Service
                  pacl->id,               // IP ID
                  0,                      // IP fragment bits and offset
                  64,                     // TTL
                  pacl->protocol,         // protocol
                  0,                      // checksum
                  pacl->source_addr,      // source ip 
                  pacl->destination_addr, // destination ip
                  NULL,                   // payload
                  0,                      // payload size
                  l,                      // libnet handle
                  0);                     // create new ptag
               if(tcp_ip == -1)
               {
                  fprintf(stderr,
                        "Can't build IP header: %s\n",libnet_geterror(l));
                  return(-1);
               }
               break;
            case IPPROTO_UDP: //17
               udp = libnet_build_udp(
                  pacl->source_port,
                  pacl->destination_port,
                  pacl->proto_h_len + pacl->payload_s,
                  0,
                  pacl->payload,
                  pacl->payload_s,
                  l,
                  0);
               if(udp == -1)
                  fprintf(stderr,"Unable to build UDP header.\n");

               udp_ip = libnet_build_ipv4(
                     (LIBNET_IPV4_H + pacl->proto_h_len + pacl->payload_s),
                     IPTOS_LOWDELAY | IPTOS_THROUGHPUT,
                     pacl->id,
                     0,
                     64,
                     pacl->protocol,
                     0,
                     pacl->source_addr,
                     pacl->destination_addr,
                     NULL,
                     0,
                     l,
                     0);
               if(udp_ip == -1)
               {
                  fprintf(stderr,
                        "Can't build IP header: %s\n",libnet_geterror(l));
                  return(-1);
               }
               break;
            case IPPROTO_IGMP: // 2
               igmp = libnet_build_igmp(
                     IGMP_MEMBERSHIP_QUERY, //type
                     0, //code
                     0, //checksum
                     pacl->destination_addr,
                     pacl->payload,
                     pacl->payload_s,
                     l,
                     0);
               if(igmp == -1)
               { 
                fprintf(stderr,"Unable to build IGMP header.\n");
                  return(-1);
               }
               igmp_ip = libnet_build_ipv4(
                     (LIBNET_IPV4_H + pacl->proto_h_len + pacl->payload_s),
                     IPTOS_LOWDELAY | IPTOS_THROUGHPUT,
                     pacl->id,
                     0,
                     64,
                     pacl->protocol,
                     0,
                     pacl->source_addr,
                     pacl->destination_addr,
                     NULL,
                     0,
                     l,
                     0);
               if(igmp_ip == -1)
               {
                  fprintf(stderr,"Can't build IP header: %s\n"
                        ,libnet_geterror(l));
                  return(-1);
               }
               break;
/*
            case IPPROTO_ESP: // 50 Encapsulating security payload
               libnet_build_ipsec_esp_hdr(
                  u_long,             // security parameter index 
                  u_long,             // ESP sequence number 
                  u_long,             // initialization vector 
                  pacl->payload,      // pointer to packet data (or NULL) 
                  pacl->payload_s,    // payload length 
                  l,         // libnet context pointer 
                  0       // packet id 
                  );
               libnet_build_ipsec_esp_ftr(
                  u_char,             // padding length 
                  u_char,             // next header pointer 
                  char *,             // authentication data 
                  pacl->payload,      // pointer to packet data (or NULL) 
                  pacl->payload_s,    // payload length 
                  l,                  // libnet context pointer 
                  0                   // packet id 
                  ); 
               break;
            case IPPROTO_AH: // 51 IPSec Authentication Header
                             // see rfc 1826 
                    libnet_build_ipsec_ah(
                  ,             // next header 
                  0,    // length:  0 indicates null authentication algorithm 
                  0,    // reserved must be set to all zeros
                  0x1000, // security parameter index : 0 indicates no security 
                  0x2ac,  // AH sequence number 
                  u_long,             // authentication data 
                  pacl->payload,     // pointer to packet data (or NULL) 
                  pacl->payload_s,    // payload length 
                  l,      // libnet context pointer 
                  0       // packet id 
                  );
               
               break;
*/
            default: // try our best to generate "other" protocols automatically
               if(pacl->protocol == 41)
               {
                  fprintf(stdout,"Unable to build IPV6 packets.\n");
                  return(-1);
               }
               other = libnet_autobuild_ipv4( 
                     (LIBNET_IPV4_H + pacl->proto_h_len + pacl->payload_s),
                     pacl->protocol,
                     pacl->destination_addr,
                     l);
               if(other == -1)
               {
                  fprintf(stderr,"Protocol %d is currently not supported.\n",
                     pacl->protocol);
                  unsupported = 1;
               }
               break;
        } // end of protocol switch

        if(!unsupported)
        {
            if((c = libnet_write(l)) == -1)
            {
                fprintf(stdout, "Write error: %s\n", libnet_geterror(l));
                return(-1);
            }
         
            if(pacl->options & VERBOSE) 
                stats(c,pacl,l);
       }
       return(0);
    }
    return(0);
}

