/*    pixilate.h
 *    Copyright 2002-2003 Kirby Kuehl (vacuum@users.sourceforge.net)
 *    This file is part of pixilate http://winfingerprint.sourceforge.net/pixilate.php
 *
 *    pixilate is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    pixilate is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *    You should have received a copy of the GNU General Public License
 *    along with pixilate; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <libnet.h>
#include <string.h>
#include <sys/types.h>
#ifndef _WIN32
#include <sys/socket.h>
#include <netinet/in.h>
#include <inttypes.h>
#include <arpa/inet.h>
#include <sys/param.h>
#include <netdb.h>
#else
#include <winsock2.h>
#define PATH_MAX _MAX_PATH
#include "win32/getopt.h"
#pragma comment (lib, "Ws2_32")
#endif
#include <errno.h>
#include <limits.h>

struct acl
{
   u_long  def_source_addr;       /* default source addreess otherwise random */
   u_short def_source_port;       /* default source port when none specified  */
   u_long  def_destination_addr;  /* destination address for 'any' ACLs       */
   u_short def_destination_port;  /* destination port when none specified     */
   u_char *def_payload;           /* default payload (NULL) if not specified  */
   u_short protocol;              /* supported protocols tcp,udp,icmp,igmp    */
   u_short proto_h_len;           /* protocol header length defined by libnet */
   u_long  source_addr;           /* current source address                   */
   u_short source_port;           /* current source port                      */
   u_long  destination_addr;      /* current destination address              */
   u_short destination_port;      /* current destination port                 */
   u_long  gateway_addr;          /* gateway address used for icmp redirect   */
   u_short icmp_type;             /* icmp type for icmp_lookup()              */
   u_short id;                    /* IP identification number                 */
   u_short seq;                   /* sequence number                          */
   int t_permits;                 /* Total permit ACLs processed              */
   int t_denies;                  /* Total deny ACLs processed                */
   int t_errors;                  /* Total ACL parsing errors                 */
   char *payload;                 /* current payload (NULL) if not specified  */
   u_long payload_s;              /* current payload size                     */
   char aclid[20];                /* name or number                           */
   char type[10];                 /* permit, deny, or compiled                */
   int options; 
   int rule;
};

/* options bit mask */
#define VERBOSE      0x0001 /* if bit set, output to stats() */
#define ROUTER       0x0002 /* if bit set, inverse mask      */
#define PERMIT_ONLY  0x0004 /* process only permit ACLs      */
#define DENY_ONLY    0x0008 /* process only deny ACLs        */
#define BOTH         0x0010 /* process both permit and deny  */
#define USE_OUTPUT   0x0020 /* redirect stdout               */

/* rule type bit mask */
#define ICMP_RULE    0x0001
#define TCP_RULE     0x0002
#define UDP_RULE     0x0004
#define IGMP_RULE    0x0008
#define IPSEC_RULE   0x0010
#define IP_RULE      0x0020

/* cisco_parser() parses Cisco PIX 6.2x - PIX 6.3x and CISCO IOS ACLs
 * other parsers should be declared seperately     */
int cisco_parser(char *inputfile, struct acl *pacl,libnet_t *l);
/* icmp_type_lookup() looks up icmp type literals and sends all icmp packets */
int cisco_icmp_type_lookup(char *literal, struct acl *pacl);
/* port_lookup() returns source or destination port */
u_short port_lookup(char *literal, struct acl *pacl);
/* protocol_lookup returns protocol type */
u_short protocol_lookup(char *literal);
int packetize(struct acl *pacl, libnet_t *l);
/* stats() displays verbose packet information     */
void stats(int c, struct acl *pacl, libnet_t *l);
/* usage() displays pixilate help */
void usage(char *progname);
