/*    stats.c
 *    Copyright 2002-2004 Kirby Kuehl (vacuum@users.sourceforge.net)
 *    This file is part of pixilate
 *    http://winfingerprint.sourceforge.net/pixilate.php
 *
 *    pixilate is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    pixilate is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *    You should have received a copy of the GNU General Public License
 *    along with pixilate; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pixilate.h"
/* The stats function displays detailed per packet information to 
 * stdout if the -v (verbose) command line option is specified.
 */

void stats(int c, struct acl *pacl, libnet_t *l)
{
    struct servent *se;
    struct protoent *pe;
    struct libnet_stats ls;
    fprintf(stdout,"Wrote         : %d byte packet.\n",c); 
    fprintf(stdout,"ACL           : %s\n",pacl->aclid);
    fprintf(stdout,"Type          : %s\n",pacl->type);
    fprintf(stdout,"Total Permits : %d\n",pacl->t_permits);
    fprintf(stdout,"Total Denies  : %d\n",pacl->t_denies);
    pe = getprotobynumber((int)pacl->protocol);
    fprintf(stdout,"Protocol      : %s %d\n",pe==NULL ? "" : pe->p_name,
   	pe==NULL ? pacl->protocol : pe->p_proto);

    if((pacl->protocol == 1) || (pacl->protocol == 2)) // ICMP or IGMP
    {
        if(pacl->protocol == 1)
        {
            switch(pacl->icmp_type)
            {
                case 0:
                    fprintf(stdout,"ICMP Type     : echo-reply 0\n");
                    break;
                case 3:
                    fprintf(stdout,"ICMP Type     : unreachable 3\n");
                    break;
                case 5:
                    fprintf(stdout,"ICMP Type     : redirect 3 (%s)\n",
                    libnet_addr2name4(pacl->gateway_addr,LIBNET_DONT_RESOLVE));
                    break;
                case 8:
                    fprintf(stdout,"ICMP Type     : echo 8\n");
                    break;
                case 11:
                    fprintf(stdout,"ICMP Type     : time-exceeded 11\n");
                    break;
                case 13:
                    fprintf(stdout,"ICMP Type     : timestamp-reply 13\n");
                    break;
                case 14:
                    fprintf(stdout,"ICMP Type     : timestamp-request 14\n");
                    break;
                case 17:
                    fprintf(stdout,"ICMP Type     : mask-request 17\n");
                    break;
                case 18:
                    fprintf(stdout,"ICMP Type     : mask-reply 18\n");
                    break;
            }      
        }
        fprintf(stdout,"Source        : %s\n",
            libnet_addr2name4(pacl->source_addr,LIBNET_DONT_RESOLVE));
        fprintf(stdout,"Destination   : %s\n",
            libnet_addr2name4(pacl->destination_addr,LIBNET_DONT_RESOLVE));
    }
    else if((pacl->protocol == 6) || (pacl->protocol == 17)) // TCP or UDP
    {
        if(pacl->protocol == 6)
            se = getservbyport(htons(pacl->destination_port),"tcp");
        if(pacl->protocol == 17)
            se = getservbyport(htons(pacl->destination_port),"udp");
        fprintf(stdout,"Source        : %s:%d\n",
            libnet_addr2name4(pacl->source_addr,LIBNET_DONT_RESOLVE),
                pacl->source_port);
        fprintf(stdout,"Destination   : %s:%d %s\n",
            libnet_addr2name4(pacl->destination_addr,LIBNET_DONT_RESOLVE),
                pacl->destination_port, se==NULL ? "" : se->s_name);
    }
    else 
    {
        fprintf(stdout,"Source        : %s\n",
            libnet_addr2name4(pacl->source_addr,LIBNET_DONT_RESOLVE));
        fprintf(stdout,"Destination   : %s\n",
            libnet_addr2name4(pacl->destination_addr,LIBNET_DONT_RESOLVE));
    }
    fprintf(stdout,"ID            : %u\n",pacl->id);
    fprintf(stdout,"SEQ           : %u\n",pacl->seq);
    fprintf(stdout,"Payload       : %s\n",pacl->payload != NULL ?
            pacl->payload : "NULL");
    libnet_stats(l, &ls);
    fprintf(stdout,"Packets sent  : %ld\n"
                   "Packet errors : %ld\n"
                   "Bytes written : %ld\n",
                   ls.packets_sent, ls.packet_errors, ls.bytes_written);
    return;
}
